@@ -142,9 +142,16 @@ class PGNParser:
 
     def __collectComment(self, data):
         index = data.find('}')
-        # TODO: Handle multiline comments
-        assert(index > 0)
-        return (data[:index+1], index+1)
+        if index < 0:
+           self.__inComment = True
+           index = data.find('\n')
+           self.__comment = self.__comment + data[:index+1]
+           return (data[:index+1], index+1)
+        else:
+            self.__comment = self.__comment + data[:index+1]
+            self.__inComment = False
+                 
+        return (self.__comment, index+1)
             
     def __extractSymbol(self, data):
         for offset in xrange(1, len(data)):
@@ -154,13 +161,12 @@ class PGNParser:
         return (data, offset)
             
     def __extractNAG(self, data):
-        index = PGNToken.NAG_CONTINUATION_CHARACTERS.find(data)
-        if index < 0:
-            raise Error('Unterminated PGN string')            
-
-            # FIXME: Should be at least one character
-            tokens.append(PGNToken(lineNumber, self.__startOffset, PGNToken.NAG, nag))
-            inNAG = False
+        index = PGNToken.NAG_CONTINUATION_CHARACTERS.find(data[1])
+        for offset in xrange(1, len(data)):            
+            if PGNToken.NAG_CONTINUATION_CHARACTERS.find(data[offset]) < 0:
+            	#FIXME: Should be at lest one character and less than $255
+                return (data[:offset], offset)
+        return (data, offset)
     
     def __extractPGNString(self, data):
         #"""Extract a PGN string.
@@ -198,10 +204,18 @@ class PGNParser:
         tokens = []
         while offset < len(line):
             c = line[offset]
-            try:
-                (tokenType, length) = self.tokens[c]
-            except KeyError:
-                raise Error('Unknown character %s' % repr(c))
+
+            if self.__inComment:
+                try:
+                    (tokenType, length) = self.tokens['{']
+                except KeyError:
+                    raise Error('Unknown character %s' % repr(c))
+            else:
+                self.__comment = ''
+                try:
+                    (tokenType, length) = self.tokens[c]
+                except KeyError:
+                    raise Error('Unknown character %s' % repr(c))
 
             startOffset = offset
             if type(length) is int:
@@ -211,7 +225,7 @@ class PGNParser:
                 (data, o) = length(line[offset:])
                 offset += o
             
-            if tokenType is not None:
+            if tokenType is not None and not self.__inComment:
                 tokens.append(PGNToken(lineNumber, startOffset, tokenType, data))
 
         return tokens
@@ -241,8 +255,10 @@ class PGNGameParser:
     
     # The move number being decoded
     __expectedMoveNumber = 0
-    __lastTokenIsMoveNumber = False
+    __prevTokenIsMoveNumber = False
     
+    __currentMoveNumber = 0
+
     # The last white move
     __whiteMove = None
     
@@ -267,7 +283,7 @@ class PGNGameParser:
             return
                 
         if token.type is PGNToken.PERIOD:
-            if self.__lastTokenIsMoveNumber is False:
+            if self.__prevTokenIsMoveNumber is False:
                 raise Error('Unexpected period on line %i:%i' % (token.lineNumber, token.characterNumber))
 
         elif token.type is PGNToken.SYMBOL:
@@ -276,10 +292,6 @@ class PGNGameParser:
                token.data == PGNToken.GAME_TERMINATE_WHITE_WIN or \
                token.data == PGNToken.GAME_TERMINATE_BLACK_WIN or \
                token.data == PGNToken.GAME_TERMINATE_DRAW:
-                # Complete any half moves
-                if self.__whiteMove is not None:
-                    self.__game.addMove(self.__whiteMove, None)
-
                 game = self.__game
                 self.__game = None
                 
@@ -290,16 +302,26 @@ class PGNGameParser:
                 # See if this is a move number or a SAN move
                 try:
                     moveNumber = int(token.data)
-                    self.__lastTokenIsMoveNumber = True
+                    self.__prevTokenIsMoveNumber = True
                     if moveNumber != self.__expectedMoveNumber:
                         raise Error('Expected move number %i, got %i on line %i:%i' % (self.__expectedMoveNumber, moveNumber, token.lineNumber, token.characterNumber))
                 except ValueError:
-                    self.__lastTokenIsMoveNumber = False
+                    self.__prevTokenIsMoveNumber = False
                     if self.__whiteMove is None:
                         self.__whiteMove = token.data
+                        move = PGNMove()
+                        move.number = self.__currentMoveNumber
+                        move.move = self.__whiteMove
+                        self.__game.addMove(move)
+                        self.__currentMoveNumber += 1
+
                     else:
-                        self.__game.addMove(self.__whiteMove, token.data)
+                        move = PGNMove()
+                        move.move = token.data
+                        move.number = self.__currentMoveNumber
+                        self.__game.addMove(move)
                         self.__whiteMove = None
+                        self.__currentMoveNumber += 1
                         self.__expectedMoveNumber += 1
 
         elif token.type is PGNToken.NAG:
@@ -313,9 +335,17 @@ class PGNGameParser:
         
         Return a game object if a game is complete otherwise None.
         """
-        
         # Ignore all comments at any time
         if token.type is PGNToken.LINE_COMMENT or token.type is PGNToken.COMMENT:
+            if self.__currentMoveNumber > 0:
+                move = self.__game.getMove(self.__currentMoveNumber)
+                move.comment = token.data[1:-1]
+            return None
+
+        if token.type is PGNToken.NAG:
+            if self.__currentMoveNumber > 0:
+                move = self.__game.getMove(self.__currentMoveNumber)
+                move.nag = token.data               
             return None
             
         if self.__state is self.STATE_IDLE:
@@ -329,7 +359,7 @@ class PGNGameParser:
             elif token.type is PGNToken.SYMBOL:
                 self.__expectedMoveNumber = 1
                 self.__whiteMove = None
-                self.__lastTokenIsMoveNumber = False
+                self.__prevTokenIsMoveNumber = False
                 self.__ravDepth = 0
                 self.__state = self.STATE_MOVETEXT
                 
@@ -372,7 +402,19 @@ class PGNGameParser:
         pass
         # Raise an error if there was a partial game
         #raise Error()
+        
+class PGNMove:
+    """
+    """
+    #
+    move       = ''
     
+    #
+    comment    = ''
+
+    #
+    nag        = ''
+
 class PGNGame:
     """
     """
@@ -447,13 +489,16 @@ class PGNGame:
         
         # Insert numbers in-between moves
         tokens = []
-        moveNumber = 1
+        moveNumber = 0
         for m in self.__moves:
-            tokens.append('%i.' % moveNumber)
+            if moveNumber % 2 == 0:
+                tokens.append('%i.' % (moveNumber / 2 + 1))
             moveNumber += 1
-            tokens.append(m[0])
-            if m[1] is not None:
-                tokens.append(m[1])
+            tokens.append(m.move)
+            if m.nag != '':
+                tokens.append(m.nag)
+            if m.comment != '':
+                tokens.append('{' + m.comment + '}')
                 
         # Add result token to the end
         tokens.append(self.__tagsByName[self.PGN_TAG_RESULT])
@@ -520,22 +565,14 @@ class PGNGame:
         except KeyError:
             return default
         
-    def addMove(self, whiteMove, blackMove):
-        self.__moves.append((whiteMove, blackMove))
-        
-    def getWhiteMove(self, moveNumber):
-        return self.__moves[moveNumber - 1][0]
-    
-    def getBlackMove(self, moveNumber):
-        return self.__moves[moveNumber - 1][1]
+    def addMove(self, move):
+        self.__moves.append(move)
+
+    def getMove(self, moveNumber):
+        return self.__moves[moveNumber - 1]
     
     def getMoves(self):
-        moves = []
-        for m in self.__moves:
-            moves.append(m[0])
-            if m[1] is not None:
-                moves.append(m[1])
-        return moves
+        return self.__moves
 
     def __str__(self):
         
@@ -672,6 +709,7 @@ class PGN:
                 
                 # Store this game and stop if only required to parse a certain number
                 if game is not None:
+                    gp = PGNGameParser()
                     self.__games.append(game)
                     gameCount += 1
 
