@@ -1539,7 +1539,6 @@ gui_draw_pause (void)
 {
   guint x, y, xadd, yadd, which;
   GdkRegion *region;
-  GdkGC *gc;
 
   if (!space->window)
     return;
@@ -1554,11 +1553,10 @@ gui_draw_pause (void)
 
       xadd = x * tile_size + xborder + (x >= size) * gap;
       yadd = y * tile_size + yborder;
-      gc = space->style->black_gc;
 
       if (which == USED)
-	gui_draw_tile (buffer, GTK_STATE_NORMAL, xadd, yadd, tiles[y][x].n,
-		       tiles[y][x].s, tiles[y][x].e, tiles[y][x].w, FALSE);
+	gui_draw_tile (buffer, GTK_STATE_NORMAL, xadd, yadd, 0, 0, 0, 0, 
+		       FALSE);
       else
 	gui_draw_socket (buffer, GTK_STATE_NORMAL, xadd, yadd);
 
